;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x3a29c0936c7c4be3e885bc089a800772 . "./code.cg2js/cg-acache.fasl")
(#x6eb8face9489bdb2088a79c39044fe90 . "./code.cg2js/cg-base.fasl")
(#xfdcfde6d83c8a122944a5c67812f1942 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xce61bb35f2dc5e408e0bf1f7f55ae141 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x1dafa0afe4f6d97bcee09b81bf649b8b . "./code.cg2js/cg-bitmap-stream.fasl")
(#x3ae1883c053abfa18b47399a57a57f8a . "./code.cg2js/cg-button.fasl")
(#xf3529d047da58705eb7e55482ab72e1a . "./code.cg2js/cg-calendar.fasl")
(#x959d55b28b0ffc3731dedac67b1c8399 . "./code.cg2js/cg-caret.fasl")
(#xd405fc3f0739b8060f3be541e962ad3c . "./code.cg2js/cg-chart-or-plot.fasl")
(#x0e9ede2e1ab73a3c775933d1b87881c0 . "./code.cg2js/cg-chart-widget.fasl")
(#x08428a428816ab8876ba741fc3d01454 . "./code.cg2js/cg-check-box.fasl")
(#xf78d2e91a4bf523445200c16cdcc04b1 . "./code.cg2js/cg-choice-list.fasl")
(#xbdbef8924e2383df26d91f228533ba48 . "./code.cg2js/cg-choose-printer.fasl")
(#x4fd5685f6d157bcd3aea51fafbeb9542 . "./code.cg2js/cg-class-grid.fasl")
(#x027640f92c6cd2632b57f0c2bc3a95a1 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x7dcdadd22aa256e0d369918e4b618b0d . "./code.cg2js/cg-class-support.fasl")
(#x6419b7007ba986aaf47afd5f8694297a . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xa615449e4bdf6863c7f1de1bb42fc913 . "./code.cg2js/cg-clipboard-stack.fasl")
(#xe2d62cc08682d73816511730e34a308a . "./code.cg2js/cg-clipboard.fasl")
(#xaa5189b280c1c0873292abf374f8576e . "./code.cg2js/cg-color-dialog.fasl")
(#x77071624419ec284548213aad6d52426 . "./code.cg2js/cg-combo-box.fasl")
(#x44aa454f514446868152fbb49d3f85fb . "./code.cg2js/cg-common-control.fasl")
(#x0e1817222cb7447001739a303237432a . "./code.cg2js/cg-comtab.fasl")
(#x007c43469dea82aa25e85a2d3d02632c . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x8f5baf60f97f78ab42b77399c9e4d1a0 . "./code.cg2js/cg-curve.fasl")
(#x07030637a077a2fc3093ba89aacc1acc . "./code.cg2js/cg-dde-utils.fasl")
(#x3a0371ca9fc5f3b6a850c5532acc9394 . "./code.cg2js/cg-dev.fasl")
(#x2037262078c92bdf9ff3c43f3bb3edb4 . "./code.cg2js/cg-dialog-item.fasl")
(#x961c6aabbea81c43b039d9981c230b3f . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x8f51f222a6793ecf482ae6443f8d251a . "./code.cg2js/cg-directory-dialog.fasl")
(#x7d13f29ac71d10da8073c20acd8c3fa6 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x2a79e9c995785db514b17b59bfe3d2f7 . "./code.cg2js/cg-drag-and-drop.fasl")
(#xae80c95343af1f7a37671fd4135ed2c4 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x004469e5998aa7dd7a71a250cc42f4ee . "./code.cg2js/cg-drawable.fasl")
(#xc4f0662a9403c041dc1e66ac290214f6 . "./code.cg2js/cg-dropping-outline.fasl")
(#xe25064c4409962530ac29eb32c9972d9 . "./code.cg2js/cg-edit-in-place.fasl")
(#xab58fe2f80f34fd6255da2f48ab10905 . "./code.cg2js/cg-editable-text.fasl")
(#xbc1f42ce006abb57a85600342ecda48a . "./code.cg2js/cg-file-dialog.fasl")
(#x405b9a4e81eecd874b1e9d6db3529c26 . "./code.cg2js/cg-fill-texture.fasl")
(#x86e078fd25caed098e4c1126da2db1b1 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8149e792f62890d83ec52b7ad08592b0 . "./code.cg2js/cg-font-dialog.fasl")
(#x2cb22626e7f40849fb491647f142251b . "./code.cg2js/cg-gesture-emulation.fasl")
(#x6431bb3c5243a31750eeddc3d31e2069 . "./code.cg2js/cg-get-pixmap.fasl")
(#x4e77ff730e14845f88c24c5ad444eb9d . "./code.cg2js/cg-get-position.fasl")
(#x50a84068280999208778b835fb0eca9f . "./code.cg2js/cg-graphics-context.fasl")
(#x90f02f0531366e37c8b3731a89b5955b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xca99487e11d1aeda6a75eb5374932eaf . "./code.cg2js/cg-grid-widget.fasl")
(#xc194857be1cfc97ffe562f16d8f34ae8 . "./code.cg2js/cg-group-box.fasl")
(#x5fd339798fa6ff77c8845807cd13c5df . "./code.cg2js/cg-header-control.fasl")
(#x98f55e01f61892fd0c7d39030c7d3fe5 . "./code.cg2js/cg-hotspot.fasl")
(#x317d836a673f48d2e988f258c71389f2 . "./code.cg2js/cg-html-dialog.fasl")
(#x5f74a550222dc3f2cfea15993b524a57 . "./code.cg2js/cg-html-widget.fasl")
(#x4b33db2907bec00b658bc4c041ae4c08 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x978249990f30eeaaf81f763e1800db89 . "./code.cg2js/cg-icon.fasl")
(#x6f82ec91bbdab245c0508aa8e283dc3b . "./code.cg2js/cg-ie.fasl")
(#x8f76ec071fa322499aa762014f24784c . "./code.cg2js/cg-intersect-posbox.fasl")
(#xeaf2bfc888aeb4f99b57d85f9f99f12d . "./code.cg2js/cg-intersect.fasl")
(#xc56e53ad1a18fa70edbfc9795cd482cc . "./code.cg2js/cg-item-list.fasl")
(#x9a1263f39d86ce194d498e6c3071c8e5 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x7bc132708e0d8d42284dbce863a5951f . "./code.cg2js/cg-lamp.fasl")
(#x0cbf6604b69d62d678137b15560f0225 . "./code.cg2js/cg-layout.fasl")
(#xef492a1e1263bed96e8e62d1277b563f . "./code.cg2js/cg-lettered-menu.fasl")
(#xf012f71fb5d1bcf9493a15f3cf75bd0e . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x5e7467292b85a50e15c6cbef51ac6a84 . "./code.cg2js/cg-lisp-text.fasl")
(#x052e69f80a489d35d0becfa679a5a7a4 . "./code.cg2js/cg-lisp-widget.fasl")
(#x238fc49fa176b3c69ae077d69e852a5a . "./code.cg2js/cg-list-view.fasl")
(#xec494f238de1bdf491195cd6b4619461 . "./code.cg2js/cg-mci.fasl")
(#x89a9e25862b131c83ea20723468d6998 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x2d0d5d5d27557f7d9ffea8a918bde578 . "./code.cg2js/cg-menu.fasl")
(#xd8cd85aee657baea08bb239f033bc09f . "./code.cg2js/cg-message-dialog.fasl")
(#x1ce07ca21c5442b7f9dce731d9350e88 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xf6ecf785c251eaeec1ab1c075f33599e . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x2c7e786712d7d66b6fbaf86869f6143b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x88fef752828e87a08ea1813aa86f75fa . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#x8487b22bfe42b4088833d86c9925189f . "./code.cg2js/cg-multi-picture-button.fasl")
(#x0487cda4f4bced8dcc3515cd07088771 . "./code.cg2js/cg-nodes.fasl")
(#x0c78c06f2786560f35c12cd45bff93df . "./code.cg2js/cg-object-editor-layout.fasl")
(#xf42d50c483bdacf64b4e2c0b8e80c382 . "./code.cg2js/cg-object-editor.fasl")
(#x2b0255a8bc3d39255b255e6ec9e7c711 . "./code.cg2js/cg-ocx.fasl")
(#x2263d2017a65c4106d07a5de2d238c86 . "./code.cg2js/cg-os-widget.fasl")
(#x4f8a3dfd1146f07e49649f70bf0f6173 . "./code.cg2js/cg-os-window.fasl")
(#x7b7f571f403b4ff09e753b025da514e5 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xacfa4877975ec7b7e875f8d5ba8279e4 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x1fd95329bd81674356ec935b3c14109c . "./code.cg2js/cg-outline.fasl")
(#x8532efd8c31b7bb4cf9ca3c36246fdb9 . "./code.cg2js/cg-palette.fasl")
(#xbac9c9b6b1fc461f08716887880d01fe . "./code.cg2js/cg-paren-matching.fasl")
(#x96a185f973d02a6220af9906bca81575 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xedf621a21030b49e8f664214100ed4c4 . "./code.cg2js/cg-picture-widget.fasl")
(#x68b9fcc32401fdefeffad3588cc24ef3 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x3e384a382cbe30ff573238001e4a953d . "./code.cg2js/cg-pixmap-printing.fasl")
(#xbae463369b14d40666b81aa713c70946 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xbde57f01dc70b9dad2ecd60049c331c1 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x9874605ee30dc9ca3bde8a386523c57c . "./code.cg2js/cg-pixmap.fasl")
(#x23bad4e4f2c23efb22b98048aeb5b931 . "./code.cg2js/cg-plot-widget.fasl")
(#x9f7d32db7ccd8a8d13f824342ee33d98 . "./code.cg2js/cg-printing.fasl")
(#x42a0ec73ac8461ad8da5ac4ab63daa61 . "./code.cg2js/cg-progress-indicator.fasl")
(#x9bd04d105f90cfa24f87a25f78439f7c . "./code.cg2js/cg-project-window.fasl")
(#xba328b80fd572f0077b8bc8ef8253b1d . "./code.cg2js/cg-property.fasl")
(#xbd52ba0aa70875f49423fe4aa9edd6af . "./code.cg2js/cg-radio-button.fasl")
(#xf52ad9e9864495aefa0255385ca88032 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x8248e9fd9d29ea672fe35e3be3a1532e . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xf8fc0eebb0466ee436f90cd0e536d886 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x17e51760675e4d5e5ea075c22f15c20d . "./code.cg2js/cg-rich-edit.fasl")
(#x0d1147f6e28cbd53f163994bafb176d8 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xa101fde5288ae319c4c3b6909e682a54 . "./code.cg2js/cg-scaling-stream.fasl")
(#x5386879d68a6a5f0f89923d0d97b0cb5 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x9618a03f28892aceac69ffabd078b77b . "./code.cg2js/cg-scroll-bar.fasl")
(#xf22c43fd785edda6d924867fc7e96235 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd70695eb369513c22735e685c7268b6d . "./code.cg2js/cg-selected-object.fasl")
(#xd9d7e6fc7adbe7b7886b1e69987d8bf0 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xf90c6d0473d431a73d7f0e7256b19558 . "./code.cg2js/cg-split-bar.fasl")
(#x3974dc2da70034fbb5c936d9fdef3a48 . "./code.cg2js/cg-static-text.fasl")
(#xc27667be369eae0bcabbdbf188a165a2 . "./code.cg2js/cg-status-bar.fasl")
(#x21e60642de8058aef1e3b8f3671818bf . "./code.cg2js/cg-string-dialog.fasl")
(#xa3be620e06025d6c088da4484f472bd9 . "./code.cg2js/cg-tab-control.fasl")
(#xdf8960eba2aca353ef40b0d3874b6591 . "./code.cg2js/cg-template-string.fasl")
(#x2ee3af8cdd5ef5ea038172a14ee1068d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x4abca64167dadf0aacdb330e4e24dc7c . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xb58a989c0aa65cb9a59bfd5a5e8d7694 . "./code.cg2js/cg-text-edit-pane.fasl")
(#x3d2dee7ae7a923611470827e113f8bac . "./code.cg2js/cg-text-or-combo.fasl")
(#xeec479a7d13d8942f19fecbfe4d719a1 . "./code.cg2js/cg-text-widget.fasl")
(#xb357fd9422bea7caa946f727fae5c399 . "./code.cg2js/cg-timer.fasl")
(#xb76f8c3ae7f3c0a5c8e774e8abfc3594 . "./code.cg2js/cg-toggling-widget.fasl")
(#xe7dbaeb122d0642c1e7d8ec2ff2f5d01 . "./code.cg2js/cg-toolbar.fasl")
(#x35a11f1528d2fd2a9cb9f63aa6118020 . "./code.cg2js/cg-tooltip.fasl")
(#x8e201f3ce04c741813427e7ddfb9fb21 . "./code.cg2js/cg-touch.fasl")
(#x42c9fae1b2692722cca008d81ec2c380 . "./code.cg2js/cg-trackbar.fasl")
(#xe62aaf881ff0b9a84a690b8cc3c756f7 . "./code.cg2js/cg-tray.fasl")
(#xaca60ea60221fb7ab911918d716ee8d0 . "./code.cg2js/cg-up-down-control.fasl")
(#xb99dd8572c346f3dafc3e2d33b567d1a . "./code.cg2js/cg-utility-dialog.fasl")
(#xfcf966d31ba2d50682fc1ec61daa5a31 . "./code.cg2js/cg-web-browser-dde.fasl")
(#xc8b2687fa9900781ff9d4aa96f35bcf6 . "./code.cg2js/cg-web-browser.fasl")
(#x33a19fc268c1f1d04928efa81402796e . "./code.cg2js/cg-wrap-string.fasl")
(#x7041f18dabc98f92b12ecdbe9423d0a4 . "./code.cg2js/cg-yes-no-list.fasl")
(#x7b807a817d49ac701123be10369e3997 . "./code.cg2js/cg-yes-no-string.fasl")
(#x3c952e3a91f4c35475144d8d5c258fc5 . "./code.cg2js/cg.fasl")
(#xb0e42058fb6aca3ba6ae480a80693bc6 . "./code.idejs/ide-acache.fasl")
(#x69ff6bbfb04dbf1a20a8dea3e7e02ac6 . "./code.idejs/ide-apropos.fasl")
(#x6f0d673b1e8b81a617358523bafbffdd . "./code.idejs/ide-base.fasl")
(#x0aa795b50d4b6cb9b5091e6f2429c30a . "./code.idejs/ide-class-browser.fasl")
(#xe884b9b92b97088e6e8909a012cbf5ec . "./code.idejs/ide-class-grapher.fasl")
(#x70e66a55f81c22d6a12fcdc35ee49798 . "./code.idejs/ide-clipboard.fasl")
(#xd658f509b768310374148b65487e8d28 . "./code.idejs/ide-dev.fasl")
(#x29e4dd1a2c0f1259bc0e891cd65b2d8a . "./code.idejs/ide-editor.fasl")
(#x6c10ca944f725d3e314a045c6243af2c . "./code.idejs/ide-find-definitions.fasl")
(#x6288e6d25d84ed5a0b01da554adeedf8 . "./code.idejs/ide-find-in-files.fasl")
(#x672d65e926a46623432399686711fcce . "./code.idejs/ide-form.fasl")
(#x7315fad50f0250256869b4c72e34836e . "./code.idejs/ide-grapher.fasl")
(#xf4f297a05d7a034b00af3d7e8b1fbdc6 . "./code.idejs/ide-inspector.fasl")
(#xdd3481ddae2da4b290f46a7f3e1b863d . "./code.idejs/ide-intro.fasl")
(#x75efa0abc57925a0307e18fade40dfff . "./code.idejs/ide-keyshort.fasl")
(#x2839465e19c9f5bd71113afe90cb82c6 . "./code.idejs/ide-navigator.fasl")
(#x84e26c3651af2e358aa0a164a5fced82 . "./code.idejs/ide-package-list.fasl")
(#x6a98733ab8a79b2b85f84ea130da2455 . "./code.idejs/ide-patch-dialog.fasl")
(#x9602261d83865e12d454fa25a0b9eda2 . "./code.idejs/ide-process-browser.fasl")
(#x3e1e4b2ceeb853d4c8bd5d7f6e82aeb3 . "./code.idejs/ide-project.fasl")
(#xc77195f7851762a290162c30058914d8 . "./code.idejs/ide-trace.fasl")
(#x84dd2f7ec9a0e1e8fccb9fde73339d47 . "./code.idejs/ide-window-list.fasl")
(#xa34d1a54f191bd299b4ace2d30a6d08d . "./code.idejs/ide.fasl")
(#xf456f3c7874f303a35ad67e1f3673be9 . "./cgjs.html")
(#x85aa0e1902236a67f67453f2c909ea34 . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
